@@ -1,3 +1,4 @@
+#if !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
 /*
  * Copyright (c) Meta Platforms, Inc. and affiliates.
  * All rights reserved.
@@ -26,3 +27,7 @@
 #if TENSORPIPE_HAS_CUDA_IPC_CHANNEL
 #include <tensorpipe/channel/cuda_ipc/factory.h>
 #endif // TENSORPIPE_HAS_CUDA_IPC_CHANNEL
+
+#else
+#error "This file should not be included when either TORCH_STABLE_ONLY or TORCH_TARGET_VERSION is defined."
+#endif  // !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
